//-----------------------------------------------------------------------------
// MK3_2560_BEEP
// Version 1.2
// 29.05.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board (oder MK2 Board)
// mit ATMega-2560 @ 16MHz (oder ATMega8)
// Sound Funktion fuer den Piepser
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_BEEP.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : @MK3 (ATMega2560)
//            Port-E (Bit3)
//            Timer-0
// Belegt : @MK2 (ATMega8)
//            Port-C (Bit0)
//            Timer-2
//
// Hinweis :
//
// > die DEFINE-Schalter "BEEP_HW_MODE" und "BEEP_QUARZ_TYP"
//   muessen zur Hardware passen !!
//-----------------------------------------------------------------------------


#include <avr/interrupt.h>

//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------

#define USE_MK3_BEEP_SONGS 1  // bei 0 werden keine internen Songs benutzt


//-----------------------------------------------------------------------------
// Hardware auswahl
//-----------------------------------------------------------------------------
#define BEEP_HW_MODE    0       // 0=MK3-Board, 1=MK1/MK2-Board/mySmartControl
#define BEEP_QUARZ_TYP  0       // 0=16MHz, 1=8MHz, 2=3.6864MHz


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_BEEP" starten
// 02. Funktion "MK3_PLAY_SOUND" spielt eine Frequenz (Hz) fuer eine Dauer (ms) ab
// 03. Funktion "MK3_PLAY_SONG" spielt einen Song aus der Liste ab
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------

typedef enum {
  SONG_TONLEITER = 0,
  SONG_FUER_ELISE
}SONG_NR;



//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_BEEP(void);	                             // zum initialisieren des Sound
void MK3_PLAY_SOUND(unsigned int frq, unsigned int dauer);   // spielt einen Ton : frq [Hz], dauer [ms]
void MK3_PLAY_SONG(SONG_NR song);                            // spielt einen Song


//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------
#if BEEP_HW_MODE==0
  // Hardware = myAVR MK3-Board
  #define MK3_DDR_BEEP  DDRE    // DDR-Register vom Sound
  #define MK3_PORT_BEEP PORTE   // Port vom Sound
  #define MK3_BEEP_OUT  0x08    // Bit vom Sound
#else
  // Hardware = myAVR MK1/MK2-Board + mySmartControl
  #define MK3_DDR_BEEP  DDRC    // DDR-Register vom Sound
  #define MK3_PORT_BEEP PORTC   // Port vom Sound
  #define MK3_BEEP_OUT  0x01    // Bit vom Sound
#endif

//-----------------------------------------------------------------------------
// interne Globale Variabeln
//-----------------------------------------------------------------------------
typedef struct {
  unsigned int frq_ist_10us;
  unsigned int frq_soll_10us;
  unsigned int time_ist_10us;
  unsigned int time_ist_100us;
  unsigned int time_soll_100us;
  bool beep_on;
  bool sound_on;
}Beep_Intern_t;
Beep_Intern_t BEEP;

//-----------------------------------------------------------------------------
void MK3_INIT_BEEP(void) {	// zum initialisieren des Sound
  // Pin als Ausgang
  // und AUS
  MK3_DDR_BEEP|=MK3_BEEP_OUT;    // als Ausgang
  MK3_PORT_BEEP&=~MK3_BEEP_OUT;  // BEEP AUS

  // alle Variablen initialisieren
  BEEP.frq_ist_10us=0;
  BEEP.frq_soll_10us=0;
  BEEP.time_ist_10us=0;
  BEEP.time_ist_100us=0;
  BEEP.time_soll_100us=0;
  BEEP.beep_on=false;
  BEEP.sound_on=false;  

  #if BEEP_HW_MODE==0
    // Hardware = myAVR MK3-Board

    // benutzter Timer : Timer-0
    // im Mode2 -> ClearTimerOnCompare-A Match

    TCCR0A=0x02;         // Mode2 (CTC)
    TCCR0B=0x00;

    TCNT0=0x00;    
    OCR0B=0x00;          // Comp-B wird nicht benutzt

    TIMSK0=0x02;         // Interrupt bei Comp-A 

    #if BEEP_QUARZ_TYP==0
      // 16MHz
      OCR0A=159;
      TCCR0B|=0x01; // Vorteiler=1, OCR=159 -> 10.00us 
    #elif BEEP_QUARZ_TYP==1
      // 8MHz
      OCR0A=79;
      TCCR0B|=0x01; // Vorteiler=1, OCR=79 -> 10.00us
    #elif BEEP_QUARZ_TYP==2
      // 3.6864MHz
      OCR0A=36;
      TCCR0B|=0x01; // Vorteiler=1, OCR=36 -> 10.04us
    #endif    
  #else
    // Hardware = myAVR MK1/MK2-Board + mySmartControl

    // benutzter Timer : Timer-2
    // im Mode2 -> ClearTimerOnCompare Match

    TCCR2=0x08;         // Mode2 (CTC)
    TCNT2=0x00;

    TIMSK=0x80;         // Interrupt bei Comp

    #if BEEP_QUARZ_TYP==0
      // 16MHz
      OCR2=159;
      TCCR2|=0x01; // Vorteiler=1, OCR=159 -> 10.00us 
    #elif BEEP_QUARZ_TYP==1
      // 8MHz
      OCR2=79;
      TCCR2|=0x01; // Vorteiler=1, OCR=79 -> 10.00us
    #elif BEEP_QUARZ_TYP==2
      // 3.6864MHz
      OCR2=36;
      TCCR2|=0x01; // Vorteiler=1, OCR=36 -> 10.04us
    #endif    
  #endif

  SREG|=0x80; // Alle Interrupts einschalten
}


//-----------------------------------------------------------------------------
// startet einen Ton mit einer bestimmten dauer
// Ton wird dann per Intterupt gestoppt
// frq in Hz
// dauer in ms
//-----------------------------------------------------------------------------
void MK3_PLAY_SOUND(unsigned int frq, unsigned int dauer)
{
  double frq_wert;

  if((frq>=10) && (frq<=10000)) {
    // frq muss zwischen 10Hz und 10kHz liegen

    if((dauer>=1) && (dauer<=5000)) {
      // dauer muss zwischen 1ms und 5sec liegen

      frq_wert=(1.0/frq)*50000;

      BEEP.frq_soll_10us=(unsigned int)(frq_wert); 
      BEEP.time_soll_100us=dauer*10;
      BEEP.sound_on=true;
      BEEP.beep_on=true;
    }
    else {
      // eintrag ueberspringen
      BEEP.frq_soll_10us=0;
      BEEP.time_soll_100us=0;
      BEEP.sound_on=false;
      BEEP.beep_on=true;
    }
  }
  else {
    // ton kann eine pause oder lied_ende sein
    if((dauer>=1) && (dauer<=5000)) {
      // dauer muss zwischen 1ms und 5sec liegen

      // pause spielen (ohne ton)
      BEEP.frq_soll_10us=0;
      BEEP.time_soll_100us=dauer*10;
      BEEP.sound_on=false;
      BEEP.beep_on=true;
    }
    else {
      // eintrag ueberspringen
      BEEP.frq_soll_10us=0;
      BEEP.time_soll_100us=0;
      BEEP.sound_on=false;
      BEEP.beep_on=true;
    }
  }  
}

#if BEEP_HW_MODE==0
// Hardware = myAVR MK3-Board
//-----------------------------------------------------------------------------
// ISR von Timer-0 Compare-A
// 1. Verwaltet den Zahler fuer die Tonfrequenz und Tondauer
// 2. Toggelt bei bedarf den Ton-Ausgang
//-----------------------------------------------------------------------------
ISR (TIMER0_COMPA_vect) {                          // ISR : Timer0 Comp-A
  // Beim Comp-A interrupt wird
  // der Timer automatisch wieder auf 0 gesetzt
  // 
  // der Interrupt wird alle 10us aufgerufen

  if(BEEP.beep_on==true) {
    BEEP.frq_ist_10us++; // 10us sind um
    BEEP.time_ist_10us++;
    if(BEEP.time_ist_10us>=10) {
      BEEP.time_ist_10us=0;
      BEEP.time_ist_100us++;
    } 

    // test ob frequenz erreicht
    if(BEEP.frq_ist_10us>=BEEP.frq_soll_10us) {
      BEEP.frq_ist_10us=0;
      if(BEEP.sound_on==true) {
        MK3_PORT_BEEP^=MK3_BEEP_OUT;  // Beep toggeln
      }
    }

    // test ob dauer erreicht
    if(BEEP.time_ist_100us>=BEEP.time_soll_100us) {
      BEEP.time_ist_100us=0;
      BEEP.frq_ist_10us=0;
      BEEP.beep_on=false;
      MK3_PORT_BEEP&=~MK3_BEEP_OUT;  // BEEP AUS
    } 
  }
}
#else
// Hardware = myAVR MK1/MK2-Board + mySmartControl
//-----------------------------------------------------------------------------
// ISR von Timer-2 Compare
// 1. Verwaltet den Zahler fuer die Tonfrequenz und Tondauer
// 2. Toggelt bei bedarf den Ton-Ausgang
//-----------------------------------------------------------------------------
ISR (TIMER2_COMP_vect) {                          // ISR : Timer2 Comp
  // Beim Comp interrupt wird
  // der Timer automatisch wieder auf 0 gesetzt
  // 
  // der Interrupt wird alle 10us aufgerufen

  if(BEEP.beep_on==true) {
    BEEP.frq_ist_10us++; // 10us sind um
    BEEP.time_ist_10us++;
    if(BEEP.time_ist_10us>=10) {
      BEEP.time_ist_10us=0;
      BEEP.time_ist_100us++;
    } 

    // test ob frequenz erreicht
    if(BEEP.frq_ist_10us>=BEEP.frq_soll_10us) {
      BEEP.frq_ist_10us=0;
      if(BEEP.sound_on==true) {
        MK3_PORT_BEEP^=MK3_BEEP_OUT;  // Beep toggeln
      }
    }

    // test ob dauer erreicht
    if(BEEP.time_ist_100us>=BEEP.time_soll_100us) {
      BEEP.time_ist_100us=0;
      BEEP.frq_ist_10us=0;
      BEEP.beep_on=false;
      MK3_PORT_BEEP&=~MK3_BEEP_OUT;  // BEEP AUS
    } 
  }
}
#endif

//-----------------------------------------------------------------------------
// tonhoehe einer Note (als Frequenz in Hz)
//-----------------------------------------------------------------------------
#if USE_MK3_BEEP_SONGS==1

#define TON_H4       TON_H3*2
#define TON_B4       TON_B3*2
#define TON_A4       TON_A3*2
#define TON_GIS4     TON_GIS3*2
#define TON_G4       TON_G3*2
#define TON_FIS4     TON_FIS3*2
#define TON_F4       TON_F3*2
#define TON_E4       TON_E3*2
#define TON_ES4      TON_ES3*2 
#define TON_D4       TON_D3*2
#define TON_CIS4     TON_CIS3*2 
#define TON_C4       TON_C3*2

#define TON_H3       TON_H2*2
#define TON_B3       TON_B2*2
#define TON_A3       TON_A2*2
#define TON_GIS3     TON_GIS2*2
#define TON_G3       TON_G2*2
#define TON_FIS3     TON_FIS2*2
#define TON_F3       TON_F2*2
#define TON_E3       TON_E2*2
#define TON_ES3      TON_ES2*2 
#define TON_D3       TON_D2*2
#define TON_CIS3     TON_CIS2*2 
#define TON_C3       TON_C2*2 

#define TON_H2       TON_H1*2
#define TON_B2       TON_B1*2
#define TON_A2       TON_A1*2
#define TON_GIS2     TON_GIS1*2
#define TON_G2       TON_G1*2
#define TON_FIS2     TON_FIS1*2
#define TON_F2       TON_F1*2
#define TON_E2       TON_E1*2
#define TON_ES2      TON_ES1*2 
#define TON_D2       TON_D1*2
#define TON_CIS2     TON_CIS1*2 
#define TON_C2       TON_C1*2 

#define TON_H1       495
#define TON_B1       475
#define TON_A1       440 // (Kammerton)
#define TON_GIS1     422
#define TON_G1       396
#define TON_FIS1     371
#define TON_F1       352
#define TON_E1       330
#define TON_ES1      317 
#define TON_D1       297
#define TON_CIS1     282 
#define TON_C1       264  
#define TON_OFF      1    // damit wird kein Ton ausgegeben
#define TON_STOP     0    // zum beeden eines Songs

#endif

//-----------------------------------------------------------------------------
// laenge einer Note (in ms)
//-----------------------------------------------------------------------------
#if USE_MK3_BEEP_SONGS==1

#define LEN_EIGHT   300   // achtel note =  0,3sec
#define LEN_QUARTER 600   // viertel note = 0,6sec
#define LEN_HALF    1200  // halbe note =   1,2sec
#define LEN_FULL    2400  // ganze note =   2,4sec
#define LEN_STOP    0     // zum beeden eines Songs

#endif


//-----------------------------------------------------------------------------
// Struktur einer Note
//-----------------------------------------------------------------------------
#if USE_MK3_BEEP_SONGS==1

typedef struct {
  unsigned int ton_hoehe;
  unsigned int ton_dauer;
}Note_t;

#endif

//-----------------------------------------------------------------------------
// Lied Nr.1 
// Tonleiter
//-----------------------------------------------------------------------------
#if USE_MK3_BEEP_SONGS==1

/*Tonleiter*/
Note_t lied1[]={
  {TON_C1,LEN_QUARTER},
  {TON_D1,LEN_QUARTER},
  {TON_E1,LEN_QUARTER},
  {TON_F1,LEN_QUARTER},
  {TON_G1,LEN_QUARTER},
  {TON_A1,LEN_QUARTER},
  {TON_H1,LEN_QUARTER},
  {TON_C2,LEN_QUARTER},
  {TON_STOP,LEN_STOP}  // beendet das Lied
};

#endif


//-----------------------------------------------------------------------------
// Lied Nr.2
// Fuer Elise
//-----------------------------------------------------------------------------
#if USE_MK3_BEEP_SONGS==1

/*Fuer Elise*/ 
Note_t lied2[]={ 
    {TON_E2,LEN_EIGHT}, 
    {TON_ES2,LEN_EIGHT}, 
    {TON_E2,LEN_EIGHT}, 
    {TON_ES2,LEN_EIGHT}, 
    {TON_E2,LEN_EIGHT},
    {TON_H1,LEN_EIGHT}, 
    {TON_D2,LEN_EIGHT}, 
    {TON_C2,LEN_EIGHT}, 
    {TON_A1,LEN_QUARTER}, 
    {TON_OFF,LEN_EIGHT}, 
    {TON_C1,LEN_EIGHT}, 
    {TON_E1,LEN_EIGHT}, 
    {TON_A1,LEN_EIGHT}, 
    {TON_H1,LEN_QUARTER}, 
    {TON_OFF,LEN_EIGHT}, 
    {TON_E1,LEN_EIGHT}, 
    {TON_GIS1,LEN_EIGHT}, 
    {TON_H1,LEN_EIGHT}, 
    {TON_C2,LEN_QUARTER}, 
    {TON_OFF,LEN_EIGHT}, 
    {TON_E2,LEN_EIGHT}, 
    {TON_ES2,LEN_EIGHT}, 
    {TON_E2,LEN_EIGHT}, 
    {TON_ES2,LEN_EIGHT}, 
    {TON_E2,LEN_EIGHT}, 
    {TON_H1,LEN_EIGHT}, 
    {TON_D2,LEN_EIGHT}, 
    {TON_C2,LEN_EIGHT}, 
    {TON_A1,LEN_QUARTER}, 
    {TON_OFF,LEN_EIGHT}, 
    {TON_C1,LEN_EIGHT}, 
    {TON_E1,LEN_EIGHT}, 
    {TON_A1,LEN_EIGHT}, 
    {TON_H1,LEN_QUARTER}, 
    {TON_OFF,LEN_EIGHT}, 
    {TON_E1,LEN_EIGHT}, 
    {TON_C2,LEN_EIGHT}, 
    {TON_H1,LEN_EIGHT}, 
    {TON_A1,LEN_HALF}, 
    {TON_STOP,LEN_STOP}  // beendet das Lied
};
#endif


//-----------------------------------------------------------------------------
// spielt einen Kompletten Song ab
// Funktion wird erst wieder verlassen, wenn Song Ende erreicht
//-----------------------------------------------------------------------------
#if USE_MK3_BEEP_SONGS==1
void MK3_PLAY_SONG(SONG_NR song)
{
  unsigned int pos=0;
  bool exit=false;
  unsigned int ton_h;
  unsigned int ton_d;

  do {
    // komplettes lied spielen
    switch(song) {
      case SONG_TONLEITER : 
        ton_h=lied1[pos].ton_hoehe;
        ton_d=lied1[pos].ton_dauer;
      break;
      case SONG_FUER_ELISE :
        ton_h=lied2[pos].ton_hoehe;
        ton_d=lied2[pos].ton_dauer;
      break;
    };


    if((ton_h==TON_STOP) || (ton_d==LEN_STOP)) {
      exit=true;
    }
    else {
      MK3_PLAY_SOUND(ton_h,ton_d);  // spiele eine note
      while(BEEP.beep_on==true);         // warte bis note fertig
      pos++;
    }

  }while(exit==false);
}
#endif

